Index: include/speex/speex.h
===================================================================
--- include/speex/speex.h	(revision 11835)
+++ include/speex/speex.h	(working copy)
@@ -388,6 +388,8 @@
  */
 int speex_decoder_ctl(void *state, int request, void *ptr);
 
+/* set assembler pointers for windows/darwin- takes encoder or decoder state */
+void speex_set_asm(int use_asm);
 
 /** Query function for mode information
  *
Index: libspeex/arch.h
===================================================================
--- libspeex/arch.h	(revision 11835)
+++ libspeex/arch.h	(working copy)
@@ -185,5 +185,30 @@
 #define LOG2_BITS_PER_CHAR 3
 
 #endif
+// way to make sse vector types more portable
+#if defined _USE_SSE
+#include <xmmintrin.h>
+#endif
 
+
+typedef union __spx_vec16_t
+{
+#if defined _USE_SSE
+	__m128 m;
+	float f[4];
+#else
+	spx_word16_t f[1];
 #endif
+} spx_vec16_t;
+
+typedef union __spx_vec32_t
+{
+#if defined _USE_SSE
+	__m128 m;
+	float f[4];
+#else
+	spx_word32_t f[1];
+#endif
+} spx_vec32_t;
+
+#endif
Index: libspeex/cb_search.c
===================================================================
--- libspeex/cb_search.c	(revision 11835)
+++ libspeex/cb_search.c	(working copy)
@@ -34,33 +34,26 @@
 #endif
 
 #include "cb_search.h"
-#include "filters.h"
 #include "stack_alloc.h"
 #include "vq.h"
 #include "misc.h"
+#include "filters.h"
 
-#ifdef _USE_SSE
-#include "cb_search_sse.h"
-#elif defined(ARM4_ASM) || defined(ARM5E_ASM)
-#include "cb_search_arm4.h"
-#elif defined(BFIN_ASM)
-#include "cb_search_bfin.h"
-#endif
-
 #ifndef OVERRIDE_COMPUTE_WEIGHTED_CODEBOOK
-static void compute_weighted_codebook(const signed char *shape_cb, const spx_word16_t *r, spx_word16_t *resp, spx_word16_t *resp2, spx_word32_t *E, int shape_cb_size, int subvect_size, char *stack)
+void compute_weighted_codebook_c(const signed char *shape_cb, const spx_word16_t *r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack)
 {
    int i, j, k;
    VARDECL(spx_word16_t *shape);
    ALLOC(shape, subvect_size, spx_word16_t);
    for (i=0;i<shape_cb_size;i++)
    {
-      spx_word16_t *res;
-      
-      res = resp+i*subvect_size;
+      spx_word16_t *res = resp+i*subvect_size;
+      spx_vec16_t *res2 = resp2+i*subvect_size;
+
       for (k=0;k<subvect_size;k++)
          shape[k] = (spx_word16_t)shape_cb[i*subvect_size+k];
-      E[i]=0;
+      
+      E[i].f[0]=0;
 
       /* Compute codeword response using convolution with impulse response */
       for(j=0;j<subvect_size;j++)
@@ -75,8 +68,10 @@
          res16 = 0.03125f*resj;
 #endif
          /* Compute codeword energy */
-         E[i]=MAC16_16(E[i],res16,res16);
+         E[i].f[0]=MAC16_16(E[i].f[0],res16,res16);
+
          res[j] = res16;
+         res2[j].f[0] = res16;
          /*printf ("%d\n", (int)res[j]);*/
       }
    }
@@ -112,13 +107,9 @@
 {
    int i,j,m,q;
    VARDECL(spx_word16_t *resp);
-#ifdef _USE_SSE
-   VARDECL(__m128 *resp2);
-   VARDECL(__m128 *E);
-#else
-   spx_word16_t *resp2;
-   VARDECL(spx_word32_t *E);
-#endif
+   VARDECL(spx_vec16_t *resp2); // not used in the c version
+   VARDECL(spx_vec32_t *E);
+
    VARDECL(spx_word16_t *t);
    VARDECL(spx_sig_t *e);
    const signed char *shape_cb;
@@ -135,13 +126,9 @@
    shape_cb = params->shape_cb;
    have_sign = params->have_sign;
    ALLOC(resp, shape_cb_size*subvect_size, spx_word16_t);
-#ifdef _USE_SSE
-   ALLOC(resp2, (shape_cb_size*subvect_size)>>2, __m128);
-   ALLOC(E, shape_cb_size>>2, __m128);
-#else
-   resp2 = resp;
-   ALLOC(E, shape_cb_size, spx_word32_t);
-#endif
+   ALLOC(resp2, shape_cb_size*subvect_size, spx_vec16_t);
+   ALLOC(E, shape_cb_size, spx_vec32_t);
+
    ALLOC(t, nsf, spx_word16_t);
    ALLOC(e, nsf, spx_sig_t);
    
@@ -254,13 +241,9 @@
 {
    int i,j,k,m,n,q;
    VARDECL(spx_word16_t *resp);
-#ifdef _USE_SSE
-   VARDECL(__m128 *resp2);
-   VARDECL(__m128 *E);
-#else
-   spx_word16_t *resp2;
-   VARDECL(spx_word32_t *E);
-#endif
+   VARDECL(spx_vec16_t *resp2);
+   VARDECL(spx_vec32_t *E);
+   
    VARDECL(spx_word16_t *t);
    VARDECL(spx_sig_t *e);
    VARDECL(spx_sig_t *r2);
@@ -307,13 +290,10 @@
    shape_cb = params->shape_cb;
    have_sign = params->have_sign;
    ALLOC(resp, shape_cb_size*subvect_size, spx_word16_t);
-#ifdef _USE_SSE
-   ALLOC(resp2, (shape_cb_size*subvect_size)>>2, __m128);
-   ALLOC(E, shape_cb_size>>2, __m128);
-#else
-   resp2 = resp;
-   ALLOC(E, shape_cb_size, spx_word32_t);
-#endif
+
+   ALLOC(resp2, shape_cb_size*subvect_size, spx_vec16_t);
+   ALLOC(E, shape_cb_size, spx_vec32_t);
+
    ALLOC(t, nsf, spx_word16_t);
    ALLOC(e, nsf, spx_sig_t);
    ALLOC(r2, nsf, spx_sig_t);
@@ -536,7 +516,10 @@
          signs[i] = speex_bits_unpack_unsigned(bits, 1);
       else
          signs[i] = 0;
+
+	  /* if abs(ind[i] is > 256 it is likely stack corruption */
       ind[i] = speex_bits_unpack_unsigned(bits, params->shape_bits);
+
    }
    /* Compute decoded excitation */
    for (i=0;i<nb_subvect;i++)
Index: libspeex/cb_search_sse.c
===================================================================
--- libspeex/cb_search_sse.c	(revision 0)
+++ libspeex/cb_search_sse.c	(revision 0)
@@ -0,0 +1,99 @@
+/* Copyright (C) 2004 Jean-Marc Valin */
+/**
+   @file cb_search_sse.c
+   @brief Fixed codebook functions (SSE version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
+#include "config.h"
+
+#ifdef _USE_SSE
+
+#include "cb_search_sse.h"
+#include "vq_sse.h"
+#include "filters.h"
+
+static inline void _spx_mm_getr_ps(__m128 U, float *__Z, float *__Y, float *__X, float *__W)
+{
+  union {
+    float __a[4];
+    __m128 __v;
+  } __u;
+  
+  __u.__v = U;
+
+  *__Z = __u.__a[0];
+  *__Y = __u.__a[1];
+  *__X = __u.__a[2];
+  *__W = __u.__a[3];
+
+}
+
+void compute_weighted_codebook_sse(const signed char *shape_cb, const spx_sig_t *_r, float *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack)
+{
+   int i, j, k;
+   __m128 resj, EE;
+   int pos =0;
+   
+   VARDECL(__m128 *r);
+   VARDECL(__m128 *shape);
+
+   ALLOC(r, subvect_size, __m128);
+   ALLOC(shape, subvect_size, __m128);
+
+   for(j=0;j<subvect_size;j++)
+   {
+      r[j] = _mm_load_ps1(_r+j);
+   }
+   for (i=0;i<shape_cb_size;i+=4)
+   {
+      float *_res = resp+i*subvect_size;
+      const signed char *_shape = shape_cb+i*subvect_size;
+      EE = _mm_setzero_ps();
+      for(j=0;j<subvect_size;j++)
+      {
+         shape[j] = _mm_setr_ps(0.03125*_shape[j], 0.03125*_shape[subvect_size+j], 0.03125*_shape[2*subvect_size+j], 0.03125*_shape[3*subvect_size+j]);
+      }
+      for(j=0;j<subvect_size;j++)
+      {
+         resj = _mm_setzero_ps();
+         for (k=0;k<=j;k++)
+            resj = _mm_add_ps(resj, _mm_mul_ps(shape[k],r[j-k]));
+         _spx_mm_getr_ps(resj, _res+j, _res+subvect_size+j, _res+2*subvect_size+j, _res+3*subvect_size+j);
+
+		 resp2[pos++].m = resj;
+
+         EE = _mm_add_ps(EE, _mm_mul_ps(resj, resj));
+      }
+      E[i>>2].m = EE;
+   }
+}
+
+#endif
Index: libspeex/cb_search_sse.h
===================================================================
--- libspeex/cb_search_sse.h	(revision 11835)
+++ libspeex/cb_search_sse.h	(working copy)
@@ -32,53 +32,14 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#ifndef _CBSEARCH_SSE_
+#define _CBSEARCH_SSE_
+
 #include <xmmintrin.h>
+#include <speex/speex_bits.h>
+#include "stack_alloc.h"
+#include "misc.h"
 
-static inline void _spx_mm_getr_ps (__m128 U, float *__Z, float *__Y, float *__X, float *__W)
-{
-  union {
-    float __a[4];
-    __m128 __v;
-  } __u;
-  
-  __u.__v = U;
+void compute_weighted_codebook_sse(const signed char *shape_cb, const spx_word16_t *_r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack);
 
-  *__Z = __u.__a[0];
-  *__Y = __u.__a[1];
-  *__X = __u.__a[2];
-  *__W = __u.__a[3];
-
-}
-
-#define OVERRIDE_COMPUTE_WEIGHTED_CODEBOOK
-static void compute_weighted_codebook(const signed char *shape_cb, const spx_sig_t *_r, float *resp, __m128 *resp2, __m128 *E, int shape_cb_size, int subvect_size, char *stack)
-{
-   int i, j, k;
-   __m128 resj, EE;
-   VARDECL(__m128 *r);
-   VARDECL(__m128 *shape);
-   ALLOC(r, subvect_size, __m128);
-   ALLOC(shape, subvect_size, __m128);
-   for(j=0;j<subvect_size;j++)
-      r[j] = _mm_load_ps1(_r+j);
-   for (i=0;i<shape_cb_size;i+=4)
-   {
-      float *_res = resp+i*subvect_size;
-      const signed char *_shape = shape_cb+i*subvect_size;
-      EE = _mm_setzero_ps();
-      for(j=0;j<subvect_size;j++)
-      {
-         shape[j] = _mm_setr_ps(0.03125*_shape[j], 0.03125*_shape[subvect_size+j], 0.03125*_shape[2*subvect_size+j], 0.03125*_shape[3*subvect_size+j]);
-      }
-      for(j=0;j<subvect_size;j++)
-      {
-         resj = _mm_setzero_ps();
-         for (k=0;k<=j;k++)
-            resj = _mm_add_ps(resj, _mm_mul_ps(shape[k],r[j-k]));
-         _spx_mm_getr_ps(resj, _res+j, _res+subvect_size+j, _res+2*subvect_size+j, _res+3*subvect_size+j);
-         *resp2++ = resj;
-         EE = _mm_add_ps(EE, _mm_mul_ps(resj, resj));
-      }
-      E[i>>2] = EE;
-   }
-}
+#endif
Index: libspeex/filters.c
===================================================================
--- libspeex/filters.c	(revision 11835)
+++ libspeex/filters.c	(working copy)
@@ -41,16 +41,7 @@
 #include "ltp.h"
 #include <math.h>
 
-#ifdef _USE_SSE
-#include "filters_sse.h"
-#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
-#include "filters_arm4.h"
-#elif defined (BFIN_ASM)
-#include "filters_bfin.h"
-#endif
 
-
-
 void bw_lpc(spx_word16_t gamma, const spx_coef_t *lpc_in, spx_coef_t *lpc_out, int order)
 {
    int i;
@@ -298,8 +289,8 @@
 
 
 #ifndef OVERRIDE_FILTER_MEM2
+void filter_mem2_c(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 #ifdef PRECISION16
-void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word16_t xi,yi,nyi;
@@ -318,7 +309,6 @@
    }
 }
 #else
-void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_sig_t xi,yi,nyi;
@@ -372,8 +362,8 @@
 
 
 #ifndef OVERRIDE_IIR_MEM2
+void iir_mem2_c(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 #ifdef PRECISION16
-void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word16_t yi,nyi;
@@ -391,7 +381,6 @@
    }
 }
 #else
-void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word32_t xi,yi,nyi;
@@ -445,8 +434,8 @@
 
 
 #ifndef OVERRIDE_FIR_MEM2
+void fir_mem2_c(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 #ifdef PRECISION16
-void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word16_t xi,yi;
@@ -464,7 +453,6 @@
    }
 }
 #else
-void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem)
 {
    int i,j;
    spx_word32_t xi,yi;
Index: libspeex/filters.h
===================================================================
--- libspeex/filters.h	(revision 11835)
+++ libspeex/filters.h	(working copy)
@@ -32,8 +32,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef FILTERS_H
-#define FILTERS_H
+#ifndef _FILTERS_H_
+#define _FILTERS_H_
 
 #include "misc.h"
 
@@ -61,11 +61,6 @@
 void qmf_decomp(const spx_word16_t *xx, const spx_word16_t *aa, spx_sig_t *, spx_sig_t *y2, int N, int M, spx_word16_t *mem, char *stack);
 void fir_mem_up(const spx_sig_t *x, const spx_word16_t *a, spx_sig_t *y, int N, int M, spx_word32_t *mem, char *stack);
 
-
-void filter_mem2(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
-void fir_mem2(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
-void iir_mem2(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
-
 void filter_mem16(const spx_word16_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_word16_t *y, int N, int ord, spx_mem_t *mem, char *stack);
 void iir_mem16(const spx_word16_t *x, const spx_coef_t *den, spx_word16_t *y, int N, int ord, spx_mem_t *mem, char *stack);
 void fir_mem16(const spx_word16_t *x, const spx_coef_t *num, spx_word16_t *y, int N, int ord, spx_mem_t *mem, char *stack);
@@ -93,4 +88,74 @@
 char *stack
 );
 
+
+#if defined (_USE_SSE)
+    typedef spx_word32_t (*inner_prod_func)(const float *, const float *, int);
+    typedef void (*pitch_xcorr_func)(const float *, const float *, float *, int, int, char *);
+    typedef void (*filter_mem2_func)(const spx_sig_t *, const spx_coef_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef void (*iir_mem2_func)(const spx_sig_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef void (*fir_mem2_func)(const spx_sig_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef void (*vq_nbest_func)(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+    typedef void (*vq_nbest_sign_func)(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+    typedef void (*compute_weighted_codebook_func)(const signed char *shape_cb, const spx_word16_t *_r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack);
+
+    filter_mem2_func fptr_fm;
+    iir_mem2_func fptr_iir;
+    fir_mem2_func fptr_fir;
+    inner_prod_func fptr_ip;
+    pitch_xcorr_func fptr_px;
+    vq_nbest_func fptr_vnb;
+    vq_nbest_sign_func fptr_vnbs;
+    compute_weighted_codebook_func fptr_cwc;
+
+    #define filter_mem2 (*fptr_fm)
+    #define iir_mem2 (*fptr_iir)
+    #define fir_mem2 (*fptr_fir)
+    #define inner_prod (*fptr_ip)
+    #define pitch_xcorr (*fptr_px)
+    #define vq_nbest (*fptr_vnb)
+    #define vq_nbest_sign (*fptr_vnbs)
+    #define compute_weighted_codebook (*fptr_cwc)
+
+#elif defined (_USE_ALTIVEC)
+
+    typedef void (*filter_mem2_func)(const spx_sig_t *, const spx_coef_t *, const spx_coef_t *, spx_sig_t *, int, int, spx_mem_t *);
+    typedef spx_word32_t (*inner_prod_func)(const float *, const float *, int);
+
+    filter_mem2_func fptr_fm;
+    inner_prod_func fptr_ip;
+
+    #define filter_mem2 (*fptr_fm)
+    #define inner_prod (*fptr_ip)
+    #define fir_mem2 fir_mem2_c
+    #define iir_mem2 iir_mem2_c
+    #define vq_nbest_sign vq_nbest_sign_c
+    #define vq_nbest vq_nbest_c
+    #define pitch_xcorr pitch_xcorr_c
+    #define compute_weighted_codebook compute_weighted_codebook_c
+
+#else
+
+    // may not get used due to overrides
+    #define filter_mem2 filter_mem2_c
+    #define inner_prod inner_prod_c
+    #define fir_mem2 fir_mem2_c
+    #define iir_mem2 iir_mem2_c
+    #define vq_nbest_sign vq_nbest_sign_c
+    #define vq_nbest vq_nbest_c
+    #define pitch_xcorr pitch_xcorr_c
+    #define compute_weighted_codebook compute_weighted_codebook_c
+
+#endif 
+
+void filter_mem2_c(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void fir_mem2_c(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void iir_mem2_c(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+spx_word32_t inner_prod_c(const spx_word16_t *x, const spx_word16_t *y, int len);
+void pitch_xcorr_c(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack);
+void vq_nbest_sign_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+void vq_nbest_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+void compute_weighted_codebook_c(const signed char *shape_cb, const spx_word16_t *_r, spx_word16_t *resp, spx_vec16_t *resp2, spx_vec32_t *E, int shape_cb_size, int subvect_size, char *stack);
+
+
 #endif
Index: libspeex/filters_altivec.c
===================================================================
--- libspeex/filters_altivec.c	(revision 0)
+++ libspeex/filters_altivec.c	(revision 0)
@@ -0,0 +1,150 @@
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file filters_altivec.h
+   @brief Various analysis/synthesis filters (altivec version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+#include "config.h"
+
+#ifdef _USE_ALTIVEC
+#include "filters_altivec.h"
+
+/*
+// original code
+    int i,j;
+	float xi,yi;
+	for (i=0;i<N;i++)
+	{
+		xi=x[i];
+		y[i] = _num[0]*xi + _mem[0];
+		yi=y[i];
+		for (j=0;j<ord-1;j++)
+		{
+			_mem[j] = _mem[j+1] + _num[j+1]*xi - _den[j+1]*yi;
+		}
+		_mem[ord-1] = _num[ord]*xi - _den[ord]*yi;
+	}
+*/
+
+void filter_mem2_altivec(const spx_sig_t *x, const spx_coef_t *_num, const spx_coef_t *_den, spx_sig_t *y, int N, int ord, spx_mem_t *_mem)
+{
+    int i,j;
+    float xi, yi;
+
+    __attribute__ ((__aligned__ (16))) float __num[20], __den[20], __mem[20];
+    __attribute__ ((__aligned__ (16))) float num[20], den[20], mem[20];
+    __attribute__ ((__aligned__ (16))) float y_aligned[4];
+    __attribute__ ((__aligned__ (16))) unsigned char permute_mask[16] = 
+        { 0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2, 3, 0, 1, 2, 3 };
+
+    // Make 16-byte aligned versions of everything.
+    memcpy((char *)__num, ((char *)_num) + 4, (ord - 1) * 4);
+    memcpy((char *)__den, ((char *)_den) + 4, (ord - 1) * 4);
+    memcpy((char *)__mem, ((char *)_mem) + 4, (ord - 1) * 4);
+
+    memcpy((char *)num, ((char *)_num), ord * 4);
+    memcpy((char *)den, ((char *)_den), ord * 4);
+    memcpy((char *)mem, ((char *)_mem), ord * 4);
+
+    __vector unsigned char zero = vec_splat_u8(0);
+    __vector unsigned char vec_permute_mask = vec_ld(0, permute_mask);
+    __vector float vec_num0, vec_mem0, vec_mem;
+    __vector float vec__mem, vec__num, vec__den;
+    __vector float vec_xi, vec_yi, vec_neg_yi;
+            
+    int pos = 0;
+    int startragged = (((unsigned long)x) / 4) & 3;
+
+    // ragged parts cant be sped up.
+    if (startragged)
+    {
+        startragged = 4 - startragged;
+        for (i = 0; i < startragged; i++)
+        {
+            xi=x[i];
+            y[i] = _num[0]*xi + _mem[0];
+            yi=y[i];
+            for (j=0;j<ord-1;j++)
+            {
+                _mem[j] = _mem[j+1] + _num[j+1]*xi - _den[j+1]*yi;
+            }
+            _mem[ord-1] = _num[ord]*xi - _den[ord]*yi;                   
+        }
+        pos = i;
+    }
+        
+    // len gets the total amount of 16-byte aligned stuff I can work on.
+    int len = N - startragged;
+    int leftovers;
+    leftovers = len & 3;
+    len -= leftovers;   // 1-3 could be left over for the end.
+
+    vec_num0 = vec_ld(0, num);
+    vec_num0 = vec_perm(vec_num0, vec_num0, vec_permute_mask);   // num0 = num[0]
+
+    for (i = pos; i < pos + len; i++)
+    {
+        vec_mem0 = vec_ld(0, mem);
+        vec_mem0 = vec_perm(vec_mem0, vec_mem0, vec_permute_mask);   // num0 = num[0]
+
+        vec_xi = vec_ld(0, &x[i]);
+        vec_yi = vec_madd(vec_num0, vec_xi, vec_mem0);
+        vec_st(vec_yi, 0, y_aligned);
+        memcpy(&y[i], y_aligned, 16);
+
+        xi=x[i];
+        yi=y[i];        
+
+        for (j=0;j<ord-1;j++)
+        {
+            _mem[j] = _mem[j+1] + _num[j+1]*xi - _den[j+1]*yi;
+        }
+        _mem[ord-1] = _num[ord]*xi - _den[ord]*yi;
+    }
+    
+    pos += len;
+
+    if (leftovers)
+    {
+        for (i = pos; i < pos + leftovers; i++)
+        {
+                xi=x[i];
+            y[i] = _num[0]*xi + _mem[0];
+            yi=y[i];
+            for (j=0;j<ord-1;j++)
+            {
+                _mem[j] = _mem[j+1] + _num[j+1]*xi - _den[j+1]*yi;
+            }
+            _mem[ord-1] = _num[ord]*xi - _den[ord]*yi;
+        }
+    }
+
+}
+#endif
Index: libspeex/filters_altivec.h
===================================================================
--- libspeex/filters_altivec.h	(revision 0)
+++ libspeex/filters_altivec.h	(revision 0)
@@ -0,0 +1,42 @@
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file filters_altivec.c
+   @brief Various analysis/synthesis filters (altivec version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
+#ifndef _FILTERS_ALTIVEC_
+#define _FILTERS_ALTIVEC_
+
+#include "misc.h"
+
+void filter_mem2_altivec(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+
+#endif
Index: libspeex/filters_sse.c
===================================================================
--- libspeex/filters_sse.c	(revision 0)
+++ libspeex/filters_sse.c	(revision 0)
@@ -0,0 +1,340 @@
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file filters_sse.c
+   @brief Various analysis/synthesis filters (SSE version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+#include "config.h"
+
+#ifdef _USE_SSE
+#include "filters_sse.h"
+#include <xmmintrin.h>
+#include "misc.h"
+
+void filter_mem2_10(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
+{
+   __m128 num[3], den[3], mem[3];
+
+   int i;
+
+   /* Copy numerator, denominator and memory to aligned xmm */
+   for (i=0;i<2;i++)
+   {
+      mem[i] = _mm_loadu_ps(_mem+4*i);
+      num[i] = _mm_loadu_ps(_num+4*i);
+      den[i] = _mm_loadu_ps(_den+4*i);
+   }
+   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
+   num[2] = _mm_setr_ps(_num[8], _num[9], 0, 0);
+   den[2] = _mm_setr_ps(_den[8], _den[9], 0, 0);
+   
+   for (i=0;i<N;i++)
+   {
+      __m128 xx;
+      __m128 yy;
+      /* Compute next filter result */
+      xx = _mm_load_ps1(x+i);
+      yy = _mm_add_ss(xx, mem[0]);
+      _mm_store_ss(y+i, yy);
+      yy = _mm_shuffle_ps(yy, yy, 0);
+      
+      /* Update memory */
+      mem[0] = _mm_move_ss(mem[0], mem[1]);
+      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+
+      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
+      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
+
+      mem[1] = _mm_move_ss(mem[1], mem[2]);
+      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+
+      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
+      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
+
+      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
+
+      mem[2] = _mm_add_ps(mem[2], _mm_mul_ps(xx, num[2]));
+      mem[2] = _mm_sub_ps(mem[2], _mm_mul_ps(yy, den[2]));
+   }
+   /* Put memory back in its place */
+   _mm_storeu_ps(_mem, mem[0]);
+   _mm_storeu_ps(_mem+4, mem[1]);
+   _mm_store_ss(_mem+8, mem[2]);
+   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
+   _mm_store_ss(_mem+9, mem[2]);
+}
+
+void filter_mem2_8(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
+{
+   __m128 num[2], den[2], mem[2];
+
+   int i;
+
+   /* Copy numerator, denominator and memory to aligned xmm */
+   for (i=0;i<2;i++)
+   {
+      mem[i] = _mm_loadu_ps(_mem+4*i);
+      num[i] = _mm_loadu_ps(_num+4*i);
+      den[i] = _mm_loadu_ps(_den+4*i);
+   }
+   
+   for (i=0;i<N;i++)
+   {
+      __m128 xx;
+      __m128 yy;
+      /* Compute next filter result */
+      xx = _mm_load_ps1(x+i);
+      yy = _mm_add_ss(xx, mem[0]);
+      _mm_store_ss(y+i, yy);
+      yy = _mm_shuffle_ps(yy, yy, 0);
+      
+      /* Update memory */
+      mem[0] = _mm_move_ss(mem[0], mem[1]);
+      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+
+      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
+      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
+
+      mem[1] = _mm_sub_ss(mem[1], mem[1]);
+      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+
+      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
+      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
+   }
+   /* Put memory back in its place */
+   _mm_storeu_ps(_mem, mem[0]);
+   _mm_storeu_ps(_mem+4, mem[1]);
+}
+
+
+void filter_mem2_sse(const spx_sig_t *x, const spx_coef_t *_num, const spx_coef_t *_den, spx_sig_t *y, int N, int ord, spx_mem_t *_mem)
+{
+   if(ord==10)
+      filter_mem2_10(x, _num, _den, y, N, ord, _mem);
+   else if (ord==8)
+      filter_mem2_8(x, _num, _den, y, N, ord, _mem);
+}
+
+
+
+void iir_mem2_10(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
+{
+   __m128 den[3], mem[3];
+
+   int i;
+
+   /* Copy numerator, denominator and memory to aligned xmm */
+   for (i=0;i<2;i++)
+   {
+      mem[i] = _mm_loadu_ps(_mem+4*i);
+      den[i] = _mm_loadu_ps(_den+4*i);
+   }
+   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
+   den[2] = _mm_setr_ps(_den[8], _den[9], 0, 0);
+   
+   for (i=0;i<N;i++)
+   {
+      __m128 xx;
+      __m128 yy;
+      /* Compute next filter result */
+      xx = _mm_load_ps1(x+i);
+      yy = _mm_add_ss(xx, mem[0]);
+      _mm_store_ss(y+i, yy);
+      yy = _mm_shuffle_ps(yy, yy, 0);
+      
+      /* Update memory */
+      mem[0] = _mm_move_ss(mem[0], mem[1]);
+      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+
+      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
+
+      mem[1] = _mm_move_ss(mem[1], mem[2]);
+      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+
+      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
+
+      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
+
+      mem[2] = _mm_sub_ps(mem[2], _mm_mul_ps(yy, den[2]));
+   }
+   /* Put memory back in its place */
+   _mm_storeu_ps(_mem, mem[0]);
+   _mm_storeu_ps(_mem+4, mem[1]);
+   _mm_store_ss(_mem+8, mem[2]);
+   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
+   _mm_store_ss(_mem+9, mem[2]);
+}
+
+
+void iir_mem2_8(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
+{
+   __m128 den[2], mem[2];
+
+   int i;
+
+   /* Copy numerator, denominator and memory to aligned xmm */
+   for (i=0;i<2;i++)
+   {
+      mem[i] = _mm_loadu_ps(_mem+4*i);
+      den[i] = _mm_loadu_ps(_den+4*i);
+   }
+   
+   for (i=0;i<N;i++)
+   {
+      __m128 xx;
+      __m128 yy;
+      /* Compute next filter result */
+      xx = _mm_load_ps1(x+i);
+      yy = _mm_add_ss(xx, mem[0]);
+      _mm_store_ss(y+i, yy);
+      yy = _mm_shuffle_ps(yy, yy, 0);
+      
+      /* Update memory */
+      mem[0] = _mm_move_ss(mem[0], mem[1]);
+      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+
+      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
+
+      mem[1] = _mm_sub_ss(mem[1], mem[1]);
+      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+
+      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
+   }
+   /* Put memory back in its place */
+   _mm_storeu_ps(_mem, mem[0]);
+   _mm_storeu_ps(_mem+4, mem[1]);
+}
+
+void iir_mem2_sse(const spx_sig_t *x, const spx_coef_t *_den, spx_sig_t *y, int N, int ord, spx_mem_t *_mem)
+{
+   if(ord==10)
+      iir_mem2_10(x, _den, y, N, ord, _mem);
+   else if (ord==8)
+      iir_mem2_8(x, _den, y, N, ord, _mem);
+}
+
+
+void fir_mem2_10(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
+{
+   __m128 num[3], mem[3];
+
+   int i;
+
+   /* Copy numerator, denominator and memory to aligned xmm */
+   for (i=0;i<2;i++)
+   {
+      mem[i] = _mm_loadu_ps(_mem+4*i);
+      num[i] = _mm_loadu_ps(_num+4*i);
+   }
+   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
+   num[2] = _mm_setr_ps(_num[8], _num[9], 0, 0);
+   
+   for (i=0;i<N;i++)
+   {
+      __m128 xx;
+      __m128 yy;
+      /* Compute next filter result */
+      xx = _mm_load_ps1(x+i);
+      yy = _mm_add_ss(xx, mem[0]);
+      _mm_store_ss(y+i, yy);
+      yy = _mm_shuffle_ps(yy, yy, 0);
+      
+      /* Update memory */
+      mem[0] = _mm_move_ss(mem[0], mem[1]);
+      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+
+      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
+
+      mem[1] = _mm_move_ss(mem[1], mem[2]);
+      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+
+      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
+
+      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
+
+      mem[2] = _mm_add_ps(mem[2], _mm_mul_ps(xx, num[2]));
+   }
+   /* Put memory back in its place */
+   _mm_storeu_ps(_mem, mem[0]);
+   _mm_storeu_ps(_mem+4, mem[1]);
+   _mm_store_ss(_mem+8, mem[2]);
+   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
+   _mm_store_ss(_mem+9, mem[2]);
+}
+
+void fir_mem2_8(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
+{
+   __m128 num[2], mem[2];
+
+   int i;
+
+   /* Copy numerator, denominator and memory to aligned xmm */
+   for (i=0;i<2;i++)
+   {
+      mem[i] = _mm_loadu_ps(_mem+4*i);
+      num[i] = _mm_loadu_ps(_num+4*i);
+   }
+   
+   for (i=0;i<N;i++)
+   {
+      __m128 xx;
+      __m128 yy;
+      /* Compute next filter result */
+      xx = _mm_load_ps1(x+i);
+      yy = _mm_add_ss(xx, mem[0]);
+      _mm_store_ss(y+i, yy);
+      yy = _mm_shuffle_ps(yy, yy, 0);
+      
+      /* Update memory */
+      mem[0] = _mm_move_ss(mem[0], mem[1]);
+      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+
+      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
+
+      mem[1] = _mm_sub_ss(mem[1], mem[1]);
+      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+
+      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
+   }
+   /* Put memory back in its place */
+   _mm_storeu_ps(_mem, mem[0]);
+   _mm_storeu_ps(_mem+4, mem[1]);
+}
+
+
+void fir_mem2_sse(const spx_sig_t *x, const spx_coef_t *_num, spx_sig_t *y, int N, int ord, spx_mem_t *_mem)
+{
+   if(ord==10)
+      fir_mem2_10(x, _num, y, N, ord, _mem);
+   else if (ord==8)
+      fir_mem2_8(x, _num, y, N, ord, _mem);
+}
+
+#endif
Index: libspeex/filters_sse.h
===================================================================
--- libspeex/filters_sse.h	(revision 11835)
+++ libspeex/filters_sse.h	(working copy)
@@ -32,305 +32,22 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <xmmintrin.h>
+#ifndef _FILTERS_SSE_
+#define _FILTERS_SSE_
 
-void filter_mem2_10(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[3], den[3], mem[3];
+#include "misc.h"
 
-   int i;
+void filter_mem2_10(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem);
+void filter_mem2_8(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem);
 
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
-   num[2] = _mm_setr_ps(_num[8], _num[9], 0, 0);
-   den[2] = _mm_setr_ps(_den[8], _den[9], 0, 0);
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
+void iir_mem2_10(const float *x, const float *_den, float *y, int N, int ord, float *_mem);
+void iir_mem2_8(const float *x, const float *_den, float *y, int N, int ord, float *_mem);
 
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
+void fir_mem2_10(const float *x, const float *_num, float *y, int N, int ord, float *_mem);
+void fir_mem2_8(const float *x, const float *_num, float *y, int N, int ord, float *_mem);
 
-      mem[1] = _mm_move_ss(mem[1], mem[2]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
+void filter_mem2_sse(const spx_sig_t *x, const spx_coef_t *num, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void iir_mem2_sse(const spx_sig_t *x, const spx_coef_t *den, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
+void fir_mem2_sse(const spx_sig_t *x, const spx_coef_t *num, spx_sig_t *y, int N, int ord, spx_mem_t *mem);
 
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-
-      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
-
-      mem[2] = _mm_add_ps(mem[2], _mm_mul_ps(xx, num[2]));
-      mem[2] = _mm_sub_ps(mem[2], _mm_mul_ps(yy, den[2]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-   _mm_store_ss(_mem+8, mem[2]);
-   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
-   _mm_store_ss(_mem+9, mem[2]);
-}
-
-void filter_mem2_8(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[2], den[2], mem[2];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
-
-      mem[1] = _mm_sub_ss(mem[1], mem[1]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-}
-
-
-#define OVERRIDE_FILTER_MEM2
-void filter_mem2(const float *x, const float *_num, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   if(ord==10)
-      filter_mem2_10(x, _num, _den, y, N, ord, _mem);
-   else if (ord==8)
-      filter_mem2_8(x, _num, _den, y, N, ord, _mem);
-}
-
-
-
-void iir_mem2_10(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 den[3], mem[3];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
-   den[2] = _mm_setr_ps(_den[8], _den[9], 0, 0);
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
-
-      mem[1] = _mm_move_ss(mem[1], mem[2]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-
-      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
-
-      mem[2] = _mm_sub_ps(mem[2], _mm_mul_ps(yy, den[2]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-   _mm_store_ss(_mem+8, mem[2]);
-   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
-   _mm_store_ss(_mem+9, mem[2]);
-}
-
-
-void iir_mem2_8(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   __m128 den[2], mem[2];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      den[i] = _mm_loadu_ps(_den+4*i);
-   }
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_sub_ps(mem[0], _mm_mul_ps(yy, den[0]));
-
-      mem[1] = _mm_sub_ss(mem[1], mem[1]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_sub_ps(mem[1], _mm_mul_ps(yy, den[1]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-}
-
-#define OVERRIDE_IIR_MEM2
-void iir_mem2(const float *x, const float *_den, float *y, int N, int ord, float *_mem)
-{
-   if(ord==10)
-      iir_mem2_10(x, _den, y, N, ord, _mem);
-   else if (ord==8)
-      iir_mem2_8(x, _den, y, N, ord, _mem);
-}
-
-
-void fir_mem2_10(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[3], mem[3];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-   }
-   mem[2] = _mm_setr_ps(_mem[8], _mem[9], 0, 0);
-   num[2] = _mm_setr_ps(_num[8], _num[9], 0, 0);
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-
-      mem[1] = _mm_move_ss(mem[1], mem[2]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-
-      mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0xfd);
-
-      mem[2] = _mm_add_ps(mem[2], _mm_mul_ps(xx, num[2]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-   _mm_store_ss(_mem+8, mem[2]);
-   mem[2] = _mm_shuffle_ps(mem[2], mem[2], 0x55);
-   _mm_store_ss(_mem+9, mem[2]);
-}
-
-void fir_mem2_8(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
-{
-   __m128 num[2], mem[2];
-
-   int i;
-
-   /* Copy numerator, denominator and memory to aligned xmm */
-   for (i=0;i<2;i++)
-   {
-      mem[i] = _mm_loadu_ps(_mem+4*i);
-      num[i] = _mm_loadu_ps(_num+4*i);
-   }
-   
-   for (i=0;i<N;i++)
-   {
-      __m128 xx;
-      __m128 yy;
-      /* Compute next filter result */
-      xx = _mm_load_ps1(x+i);
-      yy = _mm_add_ss(xx, mem[0]);
-      _mm_store_ss(y+i, yy);
-      yy = _mm_shuffle_ps(yy, yy, 0);
-      
-      /* Update memory */
-      mem[0] = _mm_move_ss(mem[0], mem[1]);
-      mem[0] = _mm_shuffle_ps(mem[0], mem[0], 0x39);
-
-      mem[0] = _mm_add_ps(mem[0], _mm_mul_ps(xx, num[0]));
-
-      mem[1] = _mm_sub_ss(mem[1], mem[1]);
-      mem[1] = _mm_shuffle_ps(mem[1], mem[1], 0x39);
-
-      mem[1] = _mm_add_ps(mem[1], _mm_mul_ps(xx, num[1]));
-   }
-   /* Put memory back in its place */
-   _mm_storeu_ps(_mem, mem[0]);
-   _mm_storeu_ps(_mem+4, mem[1]);
-}
-
-#define OVERRIDE_FIR_MEM2
-void fir_mem2(const float *x, const float *_num, float *y, int N, int ord, float *_mem)
-{
-   if(ord==10)
-      fir_mem2_10(x, _num, y, N, ord, _mem);
-   else if (ord==8)
-      fir_mem2_8(x, _num, y, N, ord, _mem);
-}
+#endif
Index: libspeex/ltp.c
===================================================================
--- libspeex/ltp.c	(revision 11835)
+++ libspeex/ltp.c	(working copy)
@@ -45,17 +45,12 @@
 #define NULL 0
 #endif
 
-
-#ifdef _USE_SSE
-#include "ltp_sse.h"
-#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
-#include "ltp_arm4.h"
-#elif defined (BFIN_ASM)
-#include "ltp_bfin.h"
-#endif
-
 #ifndef OVERRIDE_INNER_PROD
-spx_word32_t inner_prod(const spx_word16_t *x, const spx_word16_t *y, int len)
+#if defined _USE_SSE || defined _USE_ALTIVEC
+spx_word32_t inner_prod_c(const spx_word16_t *x, const spx_word16_t *y, int len)
+#else
+static inline spx_word32_t inner_prod_c(const spx_word16_t *x, const spx_word16_t *y, int len)
+#endif
 {
    spx_word32_t sum=0;
    len >>= 2;
@@ -73,9 +68,10 @@
 }
 #endif
 
+
 #ifndef OVERRIDE_PITCH_XCORR
 #if 0 /* HINT: Enable this for machines with enough registers (i.e. not x86) */
-void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
+void pitch_xcorr_c(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
 {
    int i,j;
    for (i=0;i<nb_pitch;i+=4)
@@ -138,7 +134,7 @@
 
 }
 #else
-void pitch_xcorr(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
+void pitch_xcorr_c(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack)
 {
    int i;
    for (i=0;i<nb_pitch;i++)
Index: libspeex/ltp_altivec.c
===================================================================
--- libspeex/ltp_altivec.c	(revision 0)
+++ libspeex/ltp_altivec.c	(revision 0)
@@ -0,0 +1,157 @@
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file ltp_altivec.c
+   @brief Long-Term Prediction functions (altivec version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+#include "config.h"
+#ifdef _USE_ALTIVEC
+
+#include "ltp_altivec.h"
+
+spx_word32_t inner_prod_altivec(const float *a, const float *b, int len)
+{
+	int i;
+    float sum;
+
+    int a_aligned = (((unsigned long)a) & 15) ? 0 : 1;
+    int b_aligned = (((unsigned long)b) & 15) ? 0 : 1;
+
+    __vector float MSQa, LSQa, MSQb, LSQb;
+    __vector unsigned char maska, maskb;
+    __vector float vec_a, vec_b;
+    __vector float vec_result;
+
+    vec_result = (__vector float)vec_splat_u8(0);
+
+    if ((!a_aligned) && (!b_aligned))
+    {
+        // This (unfortunately) is the common case.
+        maska = vec_lvsl(0, a);
+        maskb = vec_lvsl(0, b);
+        MSQa = vec_ld(0, a);
+        MSQb = vec_ld(0, b);
+
+        for (i = 0; i < len; i+=8)
+        {
+            a += 4;
+            LSQa = vec_ld(0, a);
+            vec_a = vec_perm(MSQa, LSQa, maska);
+
+            b += 4;
+            LSQb = vec_ld(0, b);
+            vec_b = vec_perm(MSQb, LSQb, maskb);
+
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+
+            a += 4;
+            MSQa = vec_ld(0, a);
+            vec_a = vec_perm(LSQa, MSQa, maska);
+
+            b += 4;
+            MSQb = vec_ld(0, b);
+            vec_b = vec_perm(LSQb, MSQb, maskb);
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+        }
+    }
+    else if (a_aligned && b_aligned)
+    {
+        for (i = 0; i < len; i+=8)
+        {
+            vec_a = vec_ld(0, a);
+            vec_b = vec_ld(0, b);
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+            a += 4;
+            b += 4;
+            vec_a = vec_ld(0, a);
+            vec_b = vec_ld(0, b);
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+            a += 4;
+            b += 4;
+        }
+    }
+    else if (a_aligned)
+    {
+        maskb = vec_lvsl(0, b);
+        MSQb = vec_ld(0, b);
+
+        for (i = 0; i < len; i+=8)
+        {
+            vec_a = vec_ld(0, a);
+            a += 4;
+            b += 4;
+            LSQb = vec_ld(0, b);
+            vec_b = vec_perm(MSQb, LSQb, maskb);
+
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+
+            vec_a = vec_ld(0, a);
+            a += 4;
+            b += 4;
+            MSQb = vec_ld(0, b);
+            vec_b = vec_perm(LSQb, MSQb, maskb);
+
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+        }
+    }
+    else if (b_aligned)
+    {
+        maska = vec_lvsl(0, a);
+        MSQa = vec_ld(0, a);
+        for (i = 0; i < len; i+=8)
+        {
+            a += 4;
+            LSQa = vec_ld(0, a);
+            vec_a = vec_perm(MSQa, LSQa, maska);
+
+            vec_b = vec_ld(0, b);
+            b += 4;
+
+            vec_result = vec_madd(vec_a, vec_b, vec_result);
+
+            a += 4;
+            MSQa = vec_ld(0, a);
+            vec_a = vec_perm(LSQa, MSQa, maska);
+
+            vec_b = vec_ld(0, b);
+            b += 4;
+
+            vec_result = vec_madd(vec_a, vec_b, vec_result);   
+        }
+    }
+    
+    vec_result = vec_add(vec_result, vec_sld(vec_result, vec_result, 8));
+    vec_result = vec_add(vec_result, vec_sld(vec_result, vec_result, 4));
+    vec_ste(vec_result, 0, &sum);
+
+    return sum;
+}
+
+#endif
Index: libspeex/ltp_altivec.h
===================================================================
--- libspeex/ltp_altivec.h	(revision 0)
+++ libspeex/ltp_altivec.h	(revision 0)
@@ -0,0 +1,43 @@
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file ltp_altivec.h
+   @brief Long-Term Prediction functions (altivec version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
+
+#ifndef _LTP_ALTIVEC_
+#define _LTP_ALTIVEC_
+
+#include "misc.h"
+
+spx_word32_t inner_prod_altivec(const float *a, const float *b, int len);
+
+#endif
Index: libspeex/ltp_sse.c
===================================================================
--- libspeex/ltp_sse.c	(revision 0)
+++ libspeex/ltp_sse.c	(revision 0)
@@ -0,0 +1,95 @@
+/* Copyright (C) 2002 Jean-Marc Valin */
+/**
+   @file ltp_sse.c
+   @brief Long-Term Prediction functions (SSE version)
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+#include "config.h"
+
+#ifdef _USE_SSE
+
+#include "ltp_sse.h"
+#include "stack_alloc.h"
+
+spx_word32_t inner_prod_sse(const float *a, const float *b, int len)
+{
+   int i;
+   float ret;
+   __m128 sum = _mm_setzero_ps();
+   for (i=0;i<(len>>2);i+=2)
+   {
+      sum = _mm_add_ps(sum, _mm_mul_ps(_mm_loadu_ps(a+0), _mm_loadu_ps(b+0)));
+      sum = _mm_add_ps(sum, _mm_mul_ps(_mm_loadu_ps(a+4), _mm_loadu_ps(b+4)));
+      a += 8;
+      b += 8;
+   }
+   sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
+   sum = _mm_add_ss(sum, _mm_shuffle_ps(sum, sum, 0x55));
+   _mm_store_ss(&ret, sum);
+   return ret;
+}
+
+void pitch_xcorr_sse(const float *_x, const float *_y, float *corr, int len, int nb_pitch, char *stack)
+{
+   int i, offset;
+   VARDECL(__m128 *x);
+   VARDECL(__m128 *y);
+   int N, L;
+   N = len>>2;
+   L = nb_pitch>>2;
+   ALLOC(x, N, __m128);
+   ALLOC(y, N+L, __m128);
+   for (i=0;i<N;i++)
+      x[i] = _mm_loadu_ps(_x+(i<<2));
+   for (offset=0;offset<4;offset++)
+   {
+      for (i=0;i<N+L;i++)
+         y[i] = _mm_loadu_ps(_y+(i<<2)+offset);
+      for (i=0;i<L;i++)
+      {
+         int j;
+         __m128 sum, *xx, *yy;
+         sum = _mm_setzero_ps();
+         yy = y+i;
+         xx = x;
+         for (j=0;j<N;j+=2)
+         {
+            sum = _mm_add_ps(sum, _mm_mul_ps(xx[0], yy[0]));
+            sum = _mm_add_ps(sum, _mm_mul_ps(xx[1], yy[1]));
+            xx += 2;
+            yy += 2;
+         }
+         sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
+         sum = _mm_add_ss(sum, _mm_shuffle_ps(sum, sum, 0x55));
+         _mm_store_ss(corr+nb_pitch-1-(i<<2)-offset, sum);
+      }
+   }
+}
+#endif
Index: libspeex/ltp_sse.h
===================================================================
--- libspeex/ltp_sse.h	(revision 11835)
+++ libspeex/ltp_sse.h	(working copy)
@@ -31,62 +31,14 @@
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
+#ifndef _LTP_SSE_
+#define _LTP_SSE_
 
 #include <xmmintrin.h>
+#include "misc.h"
 
-#define OVERRIDE_INNER_PROD
-float inner_prod(const float *a, const float *b, int len)
-{
-   int i;
-   float ret;
-   __m128 sum = _mm_setzero_ps();
-   for (i=0;i<(len>>2);i+=2)
-   {
-      sum = _mm_add_ps(sum, _mm_mul_ps(_mm_loadu_ps(a+0), _mm_loadu_ps(b+0)));
-      sum = _mm_add_ps(sum, _mm_mul_ps(_mm_loadu_ps(a+4), _mm_loadu_ps(b+4)));
-      a += 8;
-      b += 8;
-   }
-   sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
-   sum = _mm_add_ss(sum, _mm_shuffle_ps(sum, sum, 0x55));
-   _mm_store_ss(&ret, sum);
-   return ret;
-}
+spx_word32_t inner_prod_sse(const float *a, const float *b, int len);
+void pitch_xcorr_sse(const spx_word16_t *_x, const spx_word16_t *_y, spx_word32_t *corr, int len, int nb_pitch, char *stack);
 
-#define OVERRIDE_PITCH_XCORR
-void pitch_xcorr(const float *_x, const float *_y, float *corr, int len, int nb_pitch, char *stack)
-{
-   int i, offset;
-   VARDECL(__m128 *x);
-   VARDECL(__m128 *y);
-   int N, L;
-   N = len>>2;
-   L = nb_pitch>>2;
-   ALLOC(x, N, __m128);
-   ALLOC(y, N+L, __m128);
-   for (i=0;i<N;i++)
-      x[i] = _mm_loadu_ps(_x+(i<<2));
-   for (offset=0;offset<4;offset++)
-   {
-      for (i=0;i<N+L;i++)
-         y[i] = _mm_loadu_ps(_y+(i<<2)+offset);
-      for (i=0;i<L;i++)
-      {
-         int j;
-         __m128 sum, *xx, *yy;
-         sum = _mm_setzero_ps();
-         yy = y+i;
-         xx = x;
-         for (j=0;j<N;j+=2)
-         {
-            sum = _mm_add_ps(sum, _mm_mul_ps(xx[0], yy[0]));
-            sum = _mm_add_ps(sum, _mm_mul_ps(xx[1], yy[1]));
-            xx += 2;
-            yy += 2;
-         }
-         sum = _mm_add_ps(sum, _mm_movehl_ps(sum, sum));
-         sum = _mm_add_ss(sum, _mm_shuffle_ps(sum, sum, 0x55));
-         _mm_store_ss(corr+nb_pitch-1-(i<<2)-offset, sum);
-      }
-   }
-}
+
+#endif
Index: libspeex/Makefile.am
===================================================================
--- libspeex/Makefile.am	(revision 11835)
+++ libspeex/Makefile.am	(working copy)
@@ -17,6 +17,8 @@
 				speex_callbacks.c 	math_approx.c 	stereo.c 	preprocess.c 	smallft.c 	lbr_48k_tables.c \
 				jitter.c 	mdf.c vorbis_psy.c fftwrap.c kiss_fft.c _kiss_fft_guts.h kiss_fft.h \
 	kiss_fftr.c kiss_fftr.h window.c
+	vq_sse.c	ltp_sse.c	filters_sse.c	cb_search_sse.c \
+	filters_altivec.c	ltp_altivec.c \
 
 noinst_HEADERS = lsp.h 	nb_celp.h 	lpc.h 	lpc_bfin.h 	ltp.h 	quant_lsp.h \
 				cb_search.h 	filters.h 	stack_alloc.h 	vq.h 	vq_sse.h 	vq_arm4.h 	vq_bfin.h \
Index: libspeex/nb_celp.c
===================================================================
--- libspeex/nb_celp.c	(revision 11835)
+++ libspeex/nb_celp.c	(working copy)
@@ -64,14 +64,28 @@
 #define SUBMODE(x) st->submodes[st->submodeID]->x
 
 /* Default size for the encoder and decoder stack (can be changed at compile time).
-   This does not apply when using variable-size arrays or alloca. */
+   This does not apply when using variable-size arrays or alloca. 
+  
+  this was increased because of the SSE run time change- the reasoning is as follows
+  for wideband/narrowband, assuming the worst case is 1280 = shape_cb_size*subvect_size,
+  we are adding 1280 to the stack for resp2 for non-sse code, and 640*sizeof(__m128)/sizeof(float) 
+  or 640*4 = 2560 for the sse code.  Then we round up a bit to be safe. 
+*/
 #ifndef NB_ENC_STACK
-#define NB_ENC_STACK (8000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define NB_ENC_STACK (12000*sizeof(spx_sig_t))
+#else
+#define NB_ENC_STACK (9280*sizeof(spx_sig_t))
 #endif
+#endif
 
 #ifndef NB_DEC_STACK
-#define NB_DEC_STACK (4000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define NB_DEC_STACK (8000*sizeof(spx_sig_t))
+#else
+#define NB_DEC_STACK (5280*sizeof(spx_sig_t))
 #endif
+#endif
 
 
 #ifdef FIXED_POINT
Index: libspeex/sb_celp.c
===================================================================
--- libspeex/sb_celp.c	(revision 11835)
+++ libspeex/sb_celp.c	(working copy)
@@ -49,12 +49,20 @@
 /* Default size for the encoder and decoder stack (can be changed at compile time).
    This does not apply when using variable-size arrays or alloca. */
 #ifndef SB_ENC_STACK
-#define SB_ENC_STACK (10000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define SB_ENC_STACK (14000*sizeof(spx_sig_t))
+#else
+#define SB_ENC_STACK (11000*sizeof(spx_sig_t))
 #endif
+#endif
 
 #ifndef SB_DEC_STACK
-#define SB_DEC_STACK (6000*sizeof(spx_sig_t))
+#ifdef _USE_SSE
+#define SB_DEC_STACK (10000*sizeof(spx_sig_t))
+#else
+#define SB_DEC_STACK (7000*sizeof(spx_sig_t))
 #endif
+#endif
 
 
 #ifdef DISABLE_WIDEBAND
Index: libspeex/speex.c
===================================================================
--- libspeex/speex.c	(revision 11835)
+++ libspeex/speex.c	(working copy)
@@ -39,6 +39,29 @@
 #include "modes.h"
 #include <math.h>
 
+#include "filters.h"
+#include "ltp.h"
+
+#ifdef _USE_SSE
+#include "filters_sse.h"
+#include "cb_search_sse.h"
+#include "ltp_sse.h"
+#include "vq_sse.h"
+#elif defined _USE_ALTIVEC
+#include "filters_altivec.h"
+#include "ltp_altivec.h"
+#elif defined (ARM4_ASM) || defined(ARM5E_ASM)
+#include "filters_arm4.h"
+#include "cb_search_arm4.h"
+#include "vq_arm4.h"
+#include "ltp_arm4.h"
+#elif defined (BFIN_ASM)
+#include "filters_bfin.h"
+#include "cb_search_bfin.h"
+#include "vq_bfin.h"
+#include "ltp_bfin.h"
+#endif
+
 #ifndef NULL
 #define NULL 0
 #endif
@@ -49,11 +72,19 @@
 
 void *speex_encoder_init(const SpeexMode *mode)
 {
+#if defined(_USE_SSE) || defined(_USE_ALTIVEC)
+   if (!fptr_fm)
+	   speex_set_asm(0);
+#endif
    return mode->enc_init(mode);
 }
 
 void *speex_decoder_init(const SpeexMode *mode)
 {
+#if defined(_USE_SSE) || defined(_USE_ALTIVEC)
+   if (!fptr_fm)
+	   speex_set_asm(0);
+#endif
    return mode->dec_init(mode);
 }
 
@@ -170,7 +201,43 @@
 }
 #endif
 
+void speex_set_asm(int use_asm)
+{
+    if (use_asm)
+    {
+#ifdef _USE_SSE
+        fptr_fm = filter_mem2_sse;
+        fptr_iir = iir_mem2_sse;
+        fptr_fir = fir_mem2_sse;
+        fptr_ip = inner_prod_sse;
+        fptr_px = pitch_xcorr_sse;
+        fptr_vnb = vq_nbest_sse;
+        fptr_vnbs = vq_nbest_sign_sse;
+        fptr_cwc = compute_weighted_codebook_sse;
+#elif defined _USE_ALTIVEC
+        fptr_fm = filter_mem2_altivec;
+        fptr_ip = inner_prod_altivec;
+#endif
+    }
+    else
+    {
+#ifdef _USE_SSE
+        fptr_fm = filter_mem2_c;
+        fptr_iir = iir_mem2_c;
+        fptr_fir = fir_mem2_c;
+        fptr_ip = inner_prod_c;
+        fptr_px = pitch_xcorr_c;
+        fptr_vnb = vq_nbest_c;
+        fptr_vnbs = vq_nbest_sign_c;
+        fptr_cwc = compute_weighted_codebook_c;
+#elif defined _USE_ALTIVEC
+        fptr_fm = filter_mem2_c;
+        fptr_ip = inner_prod_c;
+#endif
 
+    }
+    return;
+}
 
 int speex_encoder_ctl(void *state, int request, void *ptr)
 {
Index: libspeex/vq.c
===================================================================
--- libspeex/vq.c	(revision 11835)
+++ libspeex/vq.c	(working copy)
@@ -38,16 +38,6 @@
 #include "stack_alloc.h"
 #include "misc.h"
 
-#ifdef _USE_SSE
-#include <xmmintrin.h>
-#include "vq_sse.h"
-#elif defined(SHORTCUTS) && (defined(ARM4_ASM) || defined(ARM5E_ASM))
-#include "vq_arm4.h"
-#elif defined(BFIN_ASM)
-#include "vq_bfin.h"
-#endif
-
-
 int scal_quant(spx_word16_t in, const spx_word16_t *boundary, int entries)
 {
    int i=0;
@@ -94,21 +84,23 @@
 }
 
 
-#ifndef OVERRIDE_VQ_NBEST
 /*Finds the indices of the n-best entries in a codebook*/
-void vq_nbest(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
+#ifndef OVERRIDE_VQ_NBEST
+void vq_nbest_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
 {
    int i,j,k,used;
+   int pos = 0;
    used = 0;
+
    for (i=0;i<entries;i++)
    {
       spx_word32_t dist=0;
       for (j=0;j<len;j++)
-         dist = MAC16_16(dist,in[j],*codebook++);
+         dist = MAC16_16(dist,in[j],codebook[pos++].f[0]);
 #ifdef FIXED_POINT
-      dist=SUB32(SHR32(E[i],1),dist);
+      dist=SUB32(SHR32(E[i].f[0],1),dist);
 #else
-      dist=.5f*E[i]-dist;
+      dist=.5f*E[i].f[0]-dist;
 #endif
       if (i<N || dist<best_dist[N-1])
       {
@@ -125,20 +117,17 @@
 }
 #endif
 
-
-
-
-#ifndef OVERRIDE_VQ_NBEST_SIGN
 /*Finds the indices of the n-best entries in a codebook with sign*/
-void vq_nbest_sign(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
+void vq_nbest_sign_c(spx_word16_t *in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
 {
    int i,j,k, sign, used;
+   int pos = 0;
    used=0;
    for (i=0;i<entries;i++)
    {
       spx_word32_t dist=0;
       for (j=0;j<len;j++)
-         dist = MAC16_16(dist,in[j],*codebook++);
+         dist = MAC16_16(dist,in[j],codebook[pos++].f[0]);
       if (dist>0)
       {
          sign=0;
@@ -148,9 +137,9 @@
          sign=1;
       }
 #ifdef FIXED_POINT
-      dist = ADD32(dist,SHR32(E[i],1));
+      dist = ADD32(dist,SHR32(E[i].f[0],1));
 #else
-      dist = ADD32(dist,.5f*E[i]);
+      dist = ADD32(dist,.5f*E[i].f[0]);
 #endif
       if (i<N || dist<best_dist[N-1])
       {
@@ -167,4 +156,3 @@
       }
    }
 }
-#endif
Index: libspeex/vq.h
===================================================================
--- libspeex/vq.h	(revision 11835)
+++ libspeex/vq.h	(working copy)
@@ -39,17 +39,6 @@
 
 int scal_quant(spx_word16_t in, const spx_word16_t *boundary, int entries);
 int scal_quant32(spx_word32_t in, const spx_word32_t *boundary, int entries);
-
 int vq_index(float *in, const float *codebook, int len, int entries);
-#ifdef _USE_SSE
-#include <xmmintrin.h>
-void vq_nbest(spx_word16_t *in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
 
-void vq_nbest_sign(spx_word16_t *in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
-#else
-void vq_nbest(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
-
-void vq_nbest_sign(spx_word16_t *in, const spx_word16_t *codebook, int len, int entries, spx_word32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
 #endif
-
-#endif
Index: libspeex/vq_sse.c
===================================================================
--- libspeex/vq_sse.c	(revision 0)
+++ libspeex/vq_sse.c	(revision 0)
@@ -0,0 +1,140 @@
+/* Copyright (C) 2004 Jean-Marc Valin */
+/**
+   @file vq_sse.c
+   @brief SSE-optimized vq routine
+*/
+/*
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+   
+   - Redistributions of source code must retain the above copyright
+   notice, this list of conditions and the following disclaimer.
+   
+   - Redistributions in binary form must reproduce the above copyright
+   notice, this list of conditions and the following disclaimer in the
+   documentation and/or other materials provided with the distribution.
+   
+   - Neither the name of the Xiph.org Foundation nor the names of its
+   contributors may be used to endorse or promote products derived from
+   this software without specific prior written permission.
+   
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+#include "config.h"
+
+#ifdef _USE_SSE
+
+#include "vq_sse.h"
+
+void vq_nbest_sse(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
+{
+   int i,j,k,used;
+   int pos = 0;
+   VARDECL(float *dist);
+   VARDECL(__m128 *in);
+   __m128 half;
+   used = 0;
+   ALLOC(dist, entries, float);
+   half = _mm_set_ps1(.5f);
+   ALLOC(in, len, __m128);
+   
+   for (i=0;i<len;i++)
+      in[i] = _mm_set_ps1(_in[i]);
+
+   for (i=0;i<entries>>2;i++)
+   {
+      __m128 d = _mm_mul_ps(E[i].m, half);
+
+	  for (j=0;j<len;j++)
+	  {
+         d = _mm_sub_ps(d, _mm_mul_ps(in[j], codebook[pos++].m));
+	  }
+
+	  _mm_storeu_ps(dist+4*i, d);
+   }
+
+   for (i=0;i<entries;i++)
+   {
+      if (i<N || dist[i]<best_dist[N-1])
+      {
+         for (k=N-1; (k >= 1) && (k > used || dist[i] < best_dist[k-1]); k--)
+         {
+            best_dist[k]=best_dist[k-1];
+            nbest[k] = nbest[k-1];
+         }
+         best_dist[k]=dist[i];
+         nbest[k]=i;
+         used++;
+      }
+   }
+}
+
+
+void vq_nbest_sign_sse(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
+{
+   int i,j,k,used;
+   int pos = 0;
+   VARDECL(float *dist);
+   VARDECL(__m128 *in);
+   __m128 half;
+   used = 0;
+   ALLOC(dist, entries, float);
+   half = _mm_set_ps1(.5f);
+   ALLOC(in, len, __m128);
+
+   for (i=0;i<len;i++)
+      in[i] = _mm_set_ps1(_in[i]);
+
+   for (i=0;i<entries>>2;i++)
+   {
+      __m128 d = _mm_setzero_ps();
+      
+	  for (j=0;j<len;j++)
+	  {
+         d = _mm_add_ps(d, _mm_mul_ps(in[j], codebook[pos++].m));
+	  }
+
+      _mm_storeu_ps(dist+4*i, d);
+   }
+
+   for (i=0;i<entries;i++)
+   {
+      int sign;
+      if (dist[i]>0)
+      {
+         sign=0;
+         dist[i]=-dist[i];
+      } else
+      {
+         sign=1;
+      }
+      dist[i] += .5f*((float*)E)[i];
+      if (i<N || dist[i]<best_dist[N-1])
+      {
+         for (k=N-1; (k >= 1) && (k > used || dist[i] < best_dist[k-1]); k--)
+         {
+            best_dist[k]=best_dist[k-1];
+            nbest[k] = nbest[k-1];
+         }
+         best_dist[k]=dist[i];
+         nbest[k]=i;
+         used++;
+         if (sign)
+            nbest[k]+=entries;
+      }
+   }
+}
+
+
+#endif
Index: libspeex/vq_sse.h
===================================================================
--- libspeex/vq_sse.h	(revision 11835)
+++ libspeex/vq_sse.h	(working copy)
@@ -32,89 +32,15 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#define OVERRIDE_VQ_NBEST
-void vq_nbest(spx_word16_t *_in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
-{
-   int i,j,k,used;
-   VARDECL(float *dist);
-   VARDECL(__m128 *in);
-   __m128 half;
-   used = 0;
-   ALLOC(dist, entries, float);
-   half = _mm_set_ps1(.5f);
-   ALLOC(in, len, __m128);
-   for (i=0;i<len;i++)
-      in[i] = _mm_set_ps1(_in[i]);
-   for (i=0;i<entries>>2;i++)
-   {
-      __m128 d = _mm_mul_ps(E[i], half);
-      for (j=0;j<len;j++)
-         d = _mm_sub_ps(d, _mm_mul_ps(in[j], *codebook++));
-      _mm_storeu_ps(dist+4*i, d);
-   }
-   for (i=0;i<entries;i++)
-   {
-      if (i<N || dist[i]<best_dist[N-1])
-      {
-         for (k=N-1; (k >= 1) && (k > used || dist[i] < best_dist[k-1]); k--)
-         {
-            best_dist[k]=best_dist[k-1];
-            nbest[k] = nbest[k-1];
-         }
-         best_dist[k]=dist[i];
-         nbest[k]=i;
-         used++;
-      }
-   }
-}
 
+#ifndef _VQNBEST_SSE_
+#define _VQNBEST_SSE_
 
+#include <xmmintrin.h>
+#include "stack_alloc.h"
+#include "misc.h"
 
+void vq_nbest_sse(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
+void vq_nbest_sign_sse(spx_word16_t *_in, const spx_vec16_t *codebook, int len, int entries, spx_vec32_t *E, int N, int *nbest, spx_word32_t *best_dist, char *stack);
 
-#define OVERRIDE_VQ_NBEST_SIGN
-void vq_nbest_sign(spx_word16_t *_in, const __m128 *codebook, int len, int entries, __m128 *E, int N, int *nbest, spx_word32_t *best_dist, char *stack)
-{
-   int i,j,k,used;
-   VARDECL(float *dist);
-   VARDECL(__m128 *in);
-   __m128 half;
-   used = 0;
-   ALLOC(dist, entries, float);
-   half = _mm_set_ps1(.5f);
-   ALLOC(in, len, __m128);
-   for (i=0;i<len;i++)
-      in[i] = _mm_set_ps1(_in[i]);
-   for (i=0;i<entries>>2;i++)
-   {
-      __m128 d = _mm_setzero_ps();
-      for (j=0;j<len;j++)
-         d = _mm_add_ps(d, _mm_mul_ps(in[j], *codebook++));
-      _mm_storeu_ps(dist+4*i, d);
-   }
-   for (i=0;i<entries;i++)
-   {
-      int sign;
-      if (dist[i]>0)
-      {
-         sign=0;
-         dist[i]=-dist[i];
-      } else
-      {
-         sign=1;
-      }
-      dist[i] += .5f*((float*)E)[i];
-      if (i<N || dist[i]<best_dist[N-1])
-      {
-         for (k=N-1; (k >= 1) && (k > used || dist[i] < best_dist[k-1]); k--)
-         {
-            best_dist[k]=best_dist[k-1];
-            nbest[k] = nbest[k-1];
-         }
-         best_dist[k]=dist[i];
-         nbest[k]=i;
-         used++;
-         if (sign)
-            nbest[k]+=entries;
-      }
-   }
-}
+#endif
